bl_info = {
    "name": "GP_FromMesh",
    "category": "GPencil",
    "version": (2, 50, 0),
    "blender": (2, 80, 0),
    "author": "Sahin Ersoz",
    "doc_url": "http://ersoz.com/SketchNCarve/SC_Main.html"
}
import bpy
from math import pi
from mathutils import Color
import bmesh
class GP_FromMesh(bpy.types.Operator):
    bl_idname = "gpencil.gp_from_mesh"
    bl_label = "GP_FromMesh"
    bl_options = {'REGISTER', 'UNDO', 'PRESET'}
    process_lines : bpy.props.BoolProperty(name = "Lines", default = 1, description="Process lines")
    process_fills : bpy.props.BoolProperty(name = "Fills", default = 1, description="Process faces as fills")
    edge_subdivide : bpy.props.IntProperty(name = "Edge Subdivide:", default = 0, min = 0, max = 5, soft_max = 5, description = "Increase subdivisions to improve Noise modifier results.")
    marked_sharp : bpy.props.BoolProperty(name = "Marked_Sharp", default = 0, description="Process edges marked 'Sharp'")
    marked_crease : bpy.props.BoolProperty(name = "Marked_Crease", default = 0, description="Process edges marked 'Crease'")
    marked_freestyle_edge : bpy.props.BoolProperty(name = "Marked_Freestyle", default = 0, description="Process edges marked 'Freestyle'")
    crease_treshold : bpy.props.FloatProperty(name = "Treshold:", default = 0.5, min = 0, max = 1, description ="Crease treshold to be considered as a 'Sharp Edge'.")
    edge_sharpness : bpy.props.FloatProperty(name = "Edge Detection Sharpness:", default = 30, min = 0, max = 180, step = 100, description ="Edge angle treshold to be considered as a 'Sharp Edge'.")
    edge_wobble : bpy.props.FloatProperty(name = "Edge Noise (Amplitude):", default = 0, min = 0, max = 50, soft_min = 0, soft_max = 1, description = "Randomize Amount")
    edge_thickness : bpy.props.FloatProperty(name = "Thickness Noise:", default = 0, min = 0, max = 1, description = "Thickness Noise")
    noise_scale : bpy.props.FloatProperty(name = "Noise Scale (Frequency):", default = 1, min = 0, max = 1, description = "Noise Scale")
    noise_randomize : bpy.props.BoolProperty(name = "Randomize (Animation)", default = 1, description="Use Random Values Over Time")
    noise_random_step : bpy.props.IntProperty(name = "Frame Step", default = 4, min = 1, max = 100, description = "Number of frames before recalculating random values again")
    edge_offset : bpy.props.FloatProperty(name = "Edge Offset:", default = 0.01, step = 1, soft_min = 0, description ="Offset Lines from the surface")
    edge_offset_even : bpy.props.BoolProperty(name = "Even Offset", default = 0, description="Offset Even")
    line_thickness : bpy.props.FloatProperty(name = "Line Thickness:", default = 2, min = 0, max = 1000, soft_max = 10, precision = 3, description = "Adjust Line Thickness")
    line_thickness_use_custom_curve : bpy.props.BoolProperty(name = "Custom Curve", default = 0, description="Use Custom Curve")
    multi_strokes : bpy.props.BoolProperty(name = "Multi Strokes", default = 0, description="Multiple Strokes modifier")
    multi_duplicates : bpy.props.IntProperty(name = "Duplicates", default = 2, min = 1, max = 50, soft_max = 10, description = "Number of strokes")
    multi_distance : bpy.props.FloatProperty(name = "Distance", default = 0.02, min = 0, max = 1, step = 1, description = "Offset of duplicates")
    multi_fade : bpy.props.BoolProperty(name = "Fade", default = 0, description="Fade")
    mesh_behind_gp : bpy.props.BoolProperty(name = "Mesh Behind GPencil", default = 1, description="Show GP Fills by retreiving the mesh surface backwards")
    outline_thickness : bpy.props.FloatProperty(name = "Outline Thickness:", default = 0.03, min = 0, max = 50, soft_min = 0, soft_max = 1, step = 1, description = "Thickness")
    even_thickness : bpy.props.BoolProperty(name = "Even Thickness", default = 0, description="Maintain thickness by Adjusting for sharp corners")
    thickness_clamp : bpy.props.FloatProperty(name = "Clamp:", default = 0.1, min = 0, max = 2, step = 1, description = "Offset Clamp based on geometry scale")
    angle_clamp : bpy.props.BoolProperty(name = "Angle Clamp", default = 0, description="Clamp thickness based on anles")
    use_vertex_group : bpy.props.BoolProperty(name = "Vertex Group", default = 0, description="Use Vertex Group")
    hide_mesh : bpy.props.BoolProperty(name = "Hide Mesh", default = 1, description="Hide Mesh Object")
    def draw(self, context):
        layout = self.layout
        box = layout.box()
        row = box.row()
        row.prop(self,"process_lines")
        row.prop(self,"process_fills")
        row = box.row()
        row.prop(self,"hide_mesh")
        if self.process_lines == 1:
            row = box.row()
            row.prop(self,"marked_sharp")
            row = box.row()
            row.prop(self,"marked_crease")
            if self.marked_crease == 1:
                row.prop(self,"crease_treshold")
            row = box.row()
            row.prop(self,"marked_freestyle_edge")
            box = layout.box()
            row = box.row()
            row.prop(self,"edge_sharpness")
            row = box.row()
            row.prop(self,"edge_subdivide")
            row = box.row()
            row.prop(self,"edge_wobble")
            if bpy.app.version >= (2, 83, 0):
                row = box.row()
                row.prop(self,"edge_thickness")
                row = box.row()
                row.prop(self,"noise_scale")
                row = box.row()
                row.prop(self,"noise_randomize")
                if self.noise_randomize == 1:
                    row.prop(self,"noise_random_step")
            row = box.row()
            row.prop(self,"edge_offset")
            scale_row = row.row()
            scale_row.scale_x = 0.8
            scale_row.prop(self,"edge_offset_even")
            row = box.row()
            row.prop(self,"line_thickness")
            scale_row = row.row()
            scale_row.scale_x = 0.8
            scale_row.prop(self,"line_thickness_use_custom_curve")
            if bpy.app.version >= (2, 83, 0):
                box = layout.box()
                row = box.row()
                row.prop(self,"multi_strokes")
                if self.multi_strokes == 1:
                    row.prop(self,"multi_duplicates")
                    row = box.row()
                    row.prop(self,"multi_distance")
                    row.prop(self,"multi_fade")
    def invoke(self, context, event):
        wm = context.window_manager
        return wm.invoke_props_dialog(self)
    def lines(self):
        ob = bpy.context.active_object
        aob = bpy.context.active_object.name
        sx,sy,sz = ob.scale
        ox, oy, oz = ob.location
        rx, ry, rz = ob.rotation_euler
        marked_sharp = self.marked_sharp
        marked_crease = self.marked_crease
        crease_treshold = self.crease_treshold
        marked_freestyle_edge = self.marked_freestyle_edge
        edge_subdivide = self.edge_subdivide
        edge_sharpness = self.edge_sharpness * pi/180
        edge_wobble = self.edge_wobble
        edge_thickness = self.edge_thickness
        noise_scale = self.noise_scale
        noise_randomize = self.noise_randomize
        noise_random_step = self.noise_random_step
        line_thickness_use_custom_curve = self.line_thickness_use_custom_curve
        multi_strokes = self.multi_strokes
        multi_duplicates = self.multi_duplicates
        multi_distance = self.multi_distance
        multi_fade = self.multi_fade
        cur_mode = bpy.context.active_object.mode
        if bpy.app.version < (2, 90, 0):
            edge_offset = -self.edge_offset
        else:
            edge_offset = self.edge_offset
        edge_offset_even = self.edge_offset_even
        line_thickness = self.line_thickness
        ob.select_set(True)
        bpy.ops.object.duplicate_move(OBJECT_OT_duplicate={"linked":False, "mode":'TRANSLATION'}, TRANSFORM_OT_translate={"value":(0, 0, 0), "orient_type":'GLOBAL', "orient_matrix":((0, 0, 0), (0, 0, 0), (0, 0, 0)), "orient_matrix_type":'GLOBAL', "constraint_axis":(False, False, False), "mirror":False, "use_proportional_edit":False, "proportional_edit_falloff":'SMOOTH', "proportional_size":1, "use_proportional_connected":False, "use_proportional_projected":False, "snap":False, "snap_target":'CLOSEST', "snap_point":(0, 0, 0), "snap_align":False, "snap_normal":(0, 0, 0), "gpencil_strokes":False, "cursor_transform":False, "texture_space":False, "remove_on_cancel":False, "release_confirm":False, "use_accurate":False})
        dob = bpy.context.active_object
        ob.select_set(False)
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='VERT')
        bpy.ops.mesh.select_all(action='SELECT')
        if edge_offset != 0:
            bpy.ops.transform.shrink_fatten(value=edge_offset, use_even_offset=edge_offset_even, mirror=True, use_proportional_edit=False, proportional_edit_falloff='SMOOTH', proportional_size=1, use_proportional_connected=False, use_proportional_projected=False, release_confirm=True)
        bpy.ops.mesh.select_all(action='DESELECT')
        bpy.ops.mesh.select_mode(type="EDGE")
        if edge_sharpness >= 0.01:
            bpy.ops.mesh.edges_select_sharp(sharpness=edge_sharpness)
        else:
            bpy.ops.mesh.select_all(action='SELECT')
        if marked_sharp == 1:
            bpy.ops.object.mode_set(mode='OBJECT')
            me = dob.data
            for e in me.edges:
                if e.use_edge_sharp:
                    e.select = True
            bpy.ops.object.mode_set(mode='EDIT')
        if marked_crease == 1:
            me = dob.data
            bpy.ops.object.mode_set(mode='OBJECT')
            for e in me.edges:
                if e.crease > crease_treshold:
                    e.select = True
                    print (e.crease)
            bpy.ops.object.mode_set(mode='EDIT')
        if marked_freestyle_edge == 1:
            bpy.ops.object.mode_set(mode='OBJECT')
            me = dob.data
            for e in me.edges:
                if e.use_freestyle_mark:
                    e.select = True
            bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.object.mode_set(mode='EDIT')
        me = dob.data
        bm = bmesh.from_edit_mesh(me)
        bm.edges.ensure_lookup_table()
        for edge in bm.edges:
            if edge.is_boundary:
                edge.select_set(True)
        bmesh.update_edit_mesh(me)
        try:
            bpy.ops.mesh.select_all(action='INVERT')
            bpy.ops.mesh.delete(type='EDGE')
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.mesh.delete(type='ONLY_FACE')
            if bpy.app.version < (3, 00, 0):
                if edge_subdivide > 0:
                    bpy.ops.mesh.select_all(action='SELECT')
                    if bpy.app.version < (2, 83, 0):
                        bpy.ops.mesh.subdivide(number_cuts=edge_subdivide, quadcorner='INNERVERT', fractal=edge_wobble, seed=0)
                    else:
                        bpy.ops.mesh.subdivide(number_cuts=edge_subdivide, quadcorner='INNERVERT', fractal=0, seed=0)
            bpy.ops.object.mode_set(mode='OBJECT')
            bpy.ops.object.convert(target='CURVE')
            bpy.ops.object.convert(target='GPENCIL')
            text = "_Copy"
            if text in ob.name:
                o_name, garbage = ob.name.split(text)
                aob = o_name
            else:
                aob = ob.name
            bpy.context.object.name = aob + "_Lines.GP"
            gp = bpy.context.object
            gp.location = ox, oy, oz
            gp.rotation_euler = rx, ry, rz
            gp.data.layers["GP_Layer"].info = "Lines"
            bpy.ops.gpencil.layer_change(layer='Lines')
            objects = bpy.data.objects
            gp.data.stroke_depth_order = '3D'
            col_name = aob+'.lines'
            if bpy.data.materials and col_name in bpy.data.materials:
                new_index = len(gp.material_slots)
                mat = bpy.data.materials[col_name]
                gp.data.materials.append(mat)
            elif len(gp.material_slots) == 1:
                gp.active_material_index = 0
                gp.active_material.name = col_name
                gp.active_material.grease_pencil.color = (0, 0, 0, 1)
                gp.active_material.grease_pencil.show_stroke = True
                gp.active_material.grease_pencil.show_fill = False
                gp.active_material.grease_pencil.mode = 'LINE'
            else:
                new_index = len(ob.material_slots)
                mat = bpy.data.materials.new(name=col_name)
                bpy.data.materials.create_gpencil_data(mat)
                gp.data.materials.append(mat)
                gp.active_material_index = new_index
                gp.active_material.grease_pencil.color = (0, 0, 0, 1)
                gp.active_material.grease_pencil.show_stroke = True
                gp.active_material.grease_pencil.show_fill = False
                gp.active_material.grease_pencil.mode = 'LINE'
        except:
            tavuk = 5
        bpy.ops.object.mode_set(mode='OBJECT')
        bpy.ops.object.select_all(action='DESELECT')
        try:
            dob.select_set(True)
            bpy.ops.object.delete(use_global=False)
        except:
            pass
        bpy.ops.object.mode_set(mode=cur_mode)
    def fills(self):
        cur_mode = bpy.context.active_object.mode
        bpy.ops.object.mode_set(mode='OBJECT')
        obj = bpy.context.active_object
        aob = bpy.context.active_object.name
        sx,sy,sz = obj.scale
        ox, oy, oz = obj.location
        rx, ry, rz = obj.rotation_euler
        text = "_Copy"
        if text in obj.name:
            o_name, garbage = obj.name.split(text)
            aob = o_name
        else:
            aob = obj.name
        gp_mesh=aob + "_Lines.GP"
        hide_mesh = self.hide_mesh
        edge_wobble = self.edge_wobble
        edge_thickness = self.edge_thickness
        mesh = obj.data
        if 1>len(mesh.materials):
            m_mat = bpy.data.materials.new(name=aob + "Mat")
            if obj.data.materials:
                obj.data.materials[0] = m_mat
            else:
                obj.data.materials.append(m_mat)
            m_mat.use_nodes = True
        if gp_mesh not in bpy.context.scene.objects:
            bpy.ops.object.gpencil_add(location=(0, 0, 0), type='EMPTY')
            bpy.context.object.data.stroke_depth_order = '3D'
            bpy.context.object.name = gp_mesh
        gpencil = bpy.context.scene.objects[gp_mesh]
        bpy.context.view_layer.objects.active = gpencil
        ob = bpy.context.active_object
        gp_name = gpencil.name
        gp_layer_name='Fills'
        if gpencil.data.layers and gp_layer_name in gpencil.data.layers:
            gpencil_layer = gpencil.data.layers[gp_layer_name]
        else:
            gpencil_layer = gpencil.data.layers.new(gp_layer_name, set_active=True)
        bpy.context.view_layer.objects.active = gpencil
        bpy.ops.gpencil.layer_change(layer='Fills')
        gp_layer = gpencil_layer
        gpencil_layer.clear()
        scene=bpy.data.scenes['Scene']
        bpy.ops.gpencil.layer_change(layer=gp_layer_name)
        gp_frame = gp_layer.frames.new(scene.frame_current)
        mat_list = []
        mat_name_list = []
        matcol_list = []
        for f in obj.data.polygons:
            v = []
            slot = obj.material_slots[f.material_index]
            mat = slot.material
            mat_name = mat.name
            try:
                if mat.use_nodes == True:
                    nodes = mat.node_tree.nodes
                    principled = next(n for n in nodes if n.type == 'BSDF_PRINCIPLED')
                    base_color = principled.inputs['Base Color']
                    value = base_color.default_value
                    color = Color( (value[0], value[1], value[2]) )
                    print (value)
                else:
                    value = mat.diffuse_color
                    color = Color( (value[0], value[1], value[2]) )
                    print (value)
            except:
                value = (0.8000, 0.0000, 0.0000)
                color = Color( (value[0], value[1], value[2]) )
                print (value)
            if mat is not None:
                mat_list.append(slot.material)
                mat_name_list.append(mat_name)
                matcol_list.append(color)
            else:
                print("No mat in slot", f.material_index)
            col_name = mat_name + "gp"
            if bpy.data.materials and col_name in bpy.data.materials:
                if col_name in ob.data.materials:
                    new_index = ob.data.materials.find(col_name)
                    mat_gp = bpy.data.materials[col_name]
                else:
                    new_index = len(ob.material_slots)
                    mat_gp = bpy.data.materials[col_name]
                    ob.data.materials.append(mat_gp)
            else:
                new_index = len(ob.material_slots)
                mat_gp = bpy.data.materials.new(name=col_name)
                bpy.data.materials.create_gpencil_data(mat_gp)
                ob.data.materials.append(mat_gp)
                ob.active_material_index = new_index
                ob.active_material.grease_pencil.color = (0, 0, 0, 1)
                ob.active_material.grease_pencil.fill_color = (value[0], value[1], value[2], 1)
                ob.active_material.grease_pencil.show_stroke = False
                ob.active_material.grease_pencil.show_fill = True
            for idx in f.vertices:
                v.append(obj.data.vertices[idx].co)
            v.append(v[0])
            vlen = (len(v))
            def draw_line(gp_frame, p0: tuple):
                gp_stroke = gp_frame.strokes.new()
                gp_stroke.display_mode = '3DSPACE'
                gp_stroke.line_width = 1
                gp_stroke.material_index = new_index
                ob = bpy.context.active_object
                gp_stroke.points.add(count=vlen)
                for i in range(vlen):
                    gp_stroke.points[i].co = v[i]
                return gp_stroke
            v1 = v[0]
            draw_line(gp_frame, (v1))
        bpy.ops.object.mode_set(mode='EDIT_GPENCIL')
        bpy.ops.gpencil.select_all(action='SELECT')
        col_name = aob+'.lines'
        if bpy.data.materials and col_name in bpy.data.materials:
            if col_name in ob.data.materials:
                new_index = ob.data.materials.find(col_name)
                ob.active_material_index = new_index
                bpy.ops.gpencil.material_select(deselect=True)
        bpy.ops.gpencil.stroke_subdivide(only_selected=False)
        bpy.ops.gpencil.stroke_simplify(factor=0.001)
        bpy.ops.object.mode_set(mode='OBJECT')
        ob.location = ox, oy, oz
        ob.rotation_euler = rx, ry, rz
        ob.scale = sx,sy,sz
        bpy.ops.object.mode_set(mode=cur_mode)
        bpy.context.scene.tool_settings.gpencil_sculpt.lock_axis = 'AXIS_Y'
        bpy.ops.gpencil.layer_change(layer='Fills')
        bpy.ops.gpencil.layer_move(type='DOWN')
        bpy.context.object.active_material_index = 0
    def execute(self, context):
        cur_mode = bpy.context.active_object.mode
        ob = bpy.context.active_object
        aob = bpy.context.active_object.name
        sx,sy,sz = ob.scale
        ox, oy, oz = ob.location
        rx, ry, rz = ob.rotation_euler
        process_lines = self.process_lines
        process_fills = self.process_fills
        hide_mesh = self.hide_mesh
        marked_sharp = self.marked_sharp
        marked_crease = self.marked_crease
        crease_treshold = self.crease_treshold
        marked_freestyle_edge = self.marked_freestyle_edge
        edge_subdivide = self.edge_subdivide
        edge_sharpness = self.edge_sharpness * pi/180
        edge_wobble = self.edge_wobble
        edge_thickness = self.edge_thickness
        noise_scale = self.noise_scale
        noise_randomize = self.noise_randomize
        noise_random_step = self.noise_random_step
        line_thickness_use_custom_curve = self.line_thickness_use_custom_curve
        multi_strokes = self.multi_strokes
        multi_duplicates = self.multi_duplicates
        multi_distance = self.multi_distance
        multi_fade = self.multi_fade
        if bpy.app.version < (2, 90, 0):
            edge_offset = -self.edge_offset
        else:
            edge_offset = self.edge_offset
        edge_offset_even = self.edge_offset_even
        line_thickness = self.line_thickness
        ob   = bpy.context.object
        gp_ob='_Lines.GP'
        if ob.type == "GPENCIL":
            if gp_ob in ob.name:
                o_name, garbage = ob.name.split(gp_ob)
                aob = o_name
        elif ob.type == "MESH":
            aob = bpy.context.active_object.name
        print(aob)
        ob = bpy.data.objects[aob]
        if aob+"_Lines.GP" not in bpy.context.scene.objects:
            bpy.ops.object.select_all(action='DESELECT')
            bpy.context.view_layer.objects.active = ob
            bpy.ops.object.mode_set(mode='OBJECT')
            ob.select_set(True)
            bpy.ops.object.duplicate_move(OBJECT_OT_duplicate={"linked":False, "mode":'TRANSLATION'}, TRANSFORM_OT_translate={"value":(0, 0, 0), "orient_type":'GLOBAL', "orient_matrix":((0, 0, 0), (0, 0, 0), (0, 0, 0)), "orient_matrix_type":'GLOBAL', "constraint_axis":(False, False, False), "mirror":False, "use_proportional_edit":False, "proportional_edit_falloff":'SMOOTH', "proportional_size":1, "use_proportional_connected":False, "use_proportional_projected":False, "snap":False, "snap_target":'CLOSEST', "snap_point":(0, 0, 0), "snap_align":False, "snap_normal":(0, 0, 0), "gpencil_strokes":False, "cursor_transform":False, "texture_space":False, "remove_on_cancel":False, "release_confirm":False, "use_accurate":False})
            dob = bpy.context.active_object
            dob.name = aob + "_Copy"
            ob.select_set(False)
            text = "_Thickness"
            text_b = "Triangulate_gpfm"
            for modifier in dob.modifiers:
                if text in modifier.name or text_b in modifier.name:
                    dob.modifiers[modifier.name].show_viewport = False
                else:
                    bpy.ops.object.modifier_apply(modifier=modifier.name)
            if process_lines == 1:
                bpy.context.view_layer.objects.active = dob
                self.lines()
                bpy.ops.object.mode_set(mode='OBJECT')
                bpy.ops.object.select_all(action='DESELECT')
            if process_fills == 1:
                bpy.ops.object.mode_set(mode='OBJECT')
                bpy.context.view_layer.objects.active = dob
                self.fills()
            bpy.ops.object.select_all(action='DESELECT')
            dob.select_set(True)
            bpy.ops.object.delete(use_global=False)
        gob = bpy.data.objects[aob+"_Lines.GP"]
        bpy.context.view_layer.objects.active = gob
        bpy.ops.object.select_all(action='DESELECT')
        if gob.type == "GPENCIL":
            if process_lines == 1:
                gob.data.layers["Lines"].hide = False
                if bpy.app.version >= (3, 00, 0):
                    if "Subdivide" not in gob.grease_pencil_modifiers:
                        bpy.ops.object.gpencil_modifier_add(type='GP_SUBDIV')
                    gob.grease_pencil_modifiers["Subdivide"].subdivision_type = 'SIMPLE'
                    gob.grease_pencil_modifiers["Subdivide"].level = edge_subdivide
                    gob.grease_pencil_modifiers["Subdivide"].layer = "Lines"
                if bpy.app.version >= (2, 83, 0):
                    if multi_strokes == 1:
                        if "MultipleStrokes" not in gob.grease_pencil_modifiers:
                            bpy.ops.object.gpencil_modifier_add(type='GP_MULTIPLY')
                        bpy.context.object.grease_pencil_modifiers["MultipleStrokes"].show_viewport = True
                        gob.grease_pencil_modifiers["MultipleStrokes"].duplicates = multi_duplicates
                        gob.grease_pencil_modifiers["MultipleStrokes"].distance = multi_distance
                        gob.grease_pencil_modifiers["MultipleStrokes"].offset = 0
                        gob.grease_pencil_modifiers["MultipleStrokes"].use_fade = multi_fade
                        gob.grease_pencil_modifiers["MultipleStrokes"].layer = "Lines"
                    else:
                        try:
                            bpy.context.object.grease_pencil_modifiers["MultipleStrokes"].show_viewport = False
                        except:
                            pass
                    if "Noise" not in gob.grease_pencil_modifiers:
                        bpy.ops.object.gpencil_modifier_add(type='GP_NOISE')
                    gob.grease_pencil_modifiers["Noise"].factor = edge_wobble
                    gob.grease_pencil_modifiers["Noise"].factor_strength = 0
                    gob.grease_pencil_modifiers["Noise"].factor_thickness = edge_thickness
                    gob.grease_pencil_modifiers["Noise"].noise_scale = noise_scale
                    try:
                        gob.grease_pencil_modifiers["Noise"].use_random = noise_randomize
                    except:
                        gob.grease_pencil_modifiers["Noise"].random = noise_randomize
                    gob.grease_pencil_modifiers["Noise"].step = noise_random_step
                    gob.grease_pencil_modifiers["Noise"].layer = "Lines"
                if "Thickness" not in gob.grease_pencil_modifiers:
                    bpy.ops.object.gpencil_modifier_add(type='GP_THICK')
                try:
                    gob.grease_pencil_modifiers["Thickness"].thickness_factor = line_thickness
                except:
                    gob.grease_pencil_modifiers["Thickness"].thickness = line_thickness
                gob.grease_pencil_modifiers["Thickness"].layer = "Lines"
                gob.grease_pencil_modifiers["Thickness"].use_custom_curve = line_thickness_use_custom_curve
            else:
                try:
                    gob.data.layers["Lines"].hide = True
                except:
                    pass
            if process_fills == 1:                    
                gob.data.layers["Fills"].hide = False
            else:
                try:
                    gob.data.layers["Fills"].hide = True
                except:
                    pass
            ob.hide_set(hide_mesh)
            ob.hide_render = hide_mesh
            try:
                bpy.context.view_layer.objects.active = bpy.data.objects[aob+"_Lines.GP"]
            except:
                pass
            bpy.ops.object.mode_set(mode=cur_mode)
        return {'FINISHED'}
def menu_func(self, context):
    self.layout.operator(GP_FromMesh.bl_idname)
def register():
    bpy.utils.register_class(GP_FromMesh)
    bpy.types.VIEW3D_MT_object.append(menu_func)
def unregister():
    bpy.utils.unregister_class(GP_FromMesh)
    bpy.types.VIEW3D_MT_object.remove(menu_func)
if __name__ == "__main__":
    register()
bpy.context.preferences.view.use_translate_new_dataname = False
